"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LambdaURI = exports.FOG_PROTO = void 0;
exports.FOG_PROTO = 'fog://';
const WILD_CARD = '*';
const ALL_INSTANCES = Number.MAX_VALUE;
const ALL_VERSIONS = WILD_CARD;
const ALL_LAMBDAS = WILD_CARD;
/**
 * @class
 */
class LambdaURI {
    /**
     * @constructor
     * @param {string} _protocol
     * @param {string} _regionCountryCode
     * @param {string} _project
     * @param {string} _lambdaId
     * @param {string} _lambdaVersion
     * @param {number} _lambdaInstance
     */
    constructor(_protocol, _regionCountryCode, _project, _lambdaId, _lambdaVersion, _lambdaInstance) {
        this._protocol = _protocol;
        this._regionCountryCode = _regionCountryCode;
        this._project = _project;
        this._lambdaId = _lambdaId;
        this._lambdaVersion = _lambdaVersion;
        this._lambdaInstance = _lambdaInstance;
    }
    /**
     * Protocol
     * @return {string}
     */
    get protocol() {
        return this._protocol;
    }
    /**
     * Region country code of this lambda
     *
     * @return {string}
     */
    get regionCountryCode() {
        return this._regionCountryCode;
    }
    /**
     * Project in which this lambda belongs
     *
     * @return {string}
     */
    get project() {
        return this._project;
    }
    /**
     * Identifier of this lambda
     *
     * @return {string}
     */
    get lambdaId() {
        return this._lambdaId;
    }
    /**
     * Version of this lambda
     *
     * @return {string}
     */
    get lambdaVersion() {
        return this._lambdaVersion;
    }
    /**
     * Instance of this lambda
     *
     * @return {number}
     */
    get lambdaInstance() {
        return this._lambdaInstance;
    }
    /**
     * Define this lambda instance number
     *
     * @param {number} value
     */
    set lambdaInstance(value) {
        this._lambdaInstance = value;
    }
    /**
     * Check if is the same project
     * @param {LambdaURI} other
     * @return {boolean}
     */
    isSameProject(other) {
        return this._project === other._project;
    }
    /**
     * Check if is the same lambda
     * @param {LambdaURI} other
     * @return {boolean}
     */
    isSameLambda(other) {
        return this.isSameProject(other) && this._lambdaId === other._lambdaId;
    }
    /**
     * Check if is the same lambda version
     * @param {LambdaURI} other
     * @return {boolean}
     */
    isSameLambdaVersion(other) {
        return this.isSameLambda(other) && this._lambdaVersion === other._lambdaVersion;
    }
    /**
     * Check if is the same lambda instance
     * @param {LambdaURI} other
     * @return {boolean}
     */
    isSameLambdaInstance(other) {
        return this.isSameLambdaVersion(other) && this._lambdaInstance === other._lambdaInstance;
    }
    /**
     * Verify if this URI's refers to lambda IDs wild card
     * @return {boolean}
     */
    isToAllLambdas() {
        return this.lambdaId === ALL_LAMBDAS;
    }
    /**
     * Verify if this URI's refers to lambda version wild card
     * @return {boolean}
     */
    isToAllLambdaVersions() {
        return this.lambdaVersion === ALL_VERSIONS;
    }
    /**
     * Verify if this URI's refers to lambda instance wild card
     * @return {boolean}
     */
    isToAllLambdaInstances() {
        return this.lambdaInstance === ALL_INSTANCES;
    }
    /**
     * String URI representation
     *
     * @return {string}
     */
    toString() {
        return `${this.protocol}${this.project}@${this.regionCountryCode}.${this.lambdaId}.${this.lambdaVersion}.${this.isToAllLambdaInstances() ? WILD_CARD : this.lambdaInstance}`;
    }
    /**
     * Create a new URI matching all lambdas
     * @return {LambdaURI}
     */
    toAllLambdas() {
        return LambdaURI.allLambdas(this.regionCountryCode, this.project);
    }
    /**
     * Create a new URI matching all lambda versions
     * @return {LambdaURI}
     */
    toAllLambdaVersions() {
        return LambdaURI.allLambdaVersions(this.regionCountryCode, this.project, this.lambdaId);
    }
    /**
     * Create a new URI matching all lambda instances
     * @return {LambdaURI}
     */
    toAllLambdaInstances() {
        return LambdaURI.allLambdaInstances(this.regionCountryCode, this.project, this.lambdaId, this.lambdaVersion);
    }
    /**
     * Match other lambdaURI with this
     * @param {LambdaURI} other
     * @return {boolean}
     */
    match(other) {
        if (!this.isSameProject(other)) {
            return false;
        }
        if (!this.isSameLambda(other)) {
            return other.isToAllLambdas();
        }
        if (!this.isSameLambdaVersion(other)) {
            return other.isToAllLambdaVersions();
        }
        if (!this.isSameLambdaInstance(other)) {
            return other.isToAllLambdaInstances();
        }
        return true;
    }
    /**
     * Create a new URI matching all lambdas
     * @param {string} regionCountryCode
     * @param {string} project
     * @return {LambdaURI}
     */
    static allLambdas(regionCountryCode, project) {
        return new LambdaURI(exports.FOG_PROTO, regionCountryCode, project, ALL_LAMBDAS, ALL_VERSIONS, ALL_INSTANCES);
    }
    /**
     * Create a new URI matching all lambda versions
     * @param {string} regionCountryCode
     * @param {string} project
     * @param {string} lambdaId
     * @return {LambdaURI}
     */
    static allLambdaVersions(regionCountryCode, project, lambdaId) {
        return new LambdaURI(exports.FOG_PROTO, regionCountryCode, project, lambdaId, ALL_VERSIONS, ALL_INSTANCES);
    }
    /**
     * Create a new URI matching all lambda instances
     * @param {string} regionCountryCode
     * @param {string} project
     * @param {string} lambdaId
     * @param {string} lambdaVersion
     * @return {LambdaURI}
     */
    static allLambdaInstances(regionCountryCode, project, lambdaId, lambdaVersion) {
        return new LambdaURI(exports.FOG_PROTO, regionCountryCode, project, lambdaId, lambdaVersion, ALL_INSTANCES);
    }
    /**
     * @param {string} uri
     *
     * @return {LambdaURI}
     * @throws TypeError If argument is not string
     * @throws Error If is an invalid URI
     */
    static parseFromString(uri) {
        if (typeof uri === 'string'
            && uri.length > 0
            && uri.length < 255) {
            let proto = '';
            if (uri.startsWith(exports.FOG_PROTO)) {
                proto = uri.substring(0, exports.FOG_PROTO.length);
            }
            const idx = uri.indexOf("@", proto.length);
            if (idx < 0) {
                throw new Error("Invalid lambda URI");
            }
            const project = uri.substring(proto.length, idx);
            const [region, id, version, instance] = uri.substring(idx + 1).split(".");
            if (region && project) {
                return new LambdaURI(exports.FOG_PROTO, region, project, id || ALL_LAMBDAS, version || ALL_VERSIONS, Number((instance && instance != WILD_CARD) ? instance : ALL_INSTANCES));
            }
            throw new Error("Invalid lambda URI");
        }
        throw new TypeError("URI is invalid data type/length.");
    }
    /**
     * Instance from lambda object
     *
     * @param {Object} lambda
     * @param {string} regionCountryCode
     * @return {LambdaURI}
     * @throws TypeError If argument is not object
     * @throws Error If object does have the methods
     */
    static parseFromLambda(lambda, regionCountryCode) {
        if (typeof lambda === 'object') {
            try {
                return new LambdaURI(exports.FOG_PROTO, regionCountryCode, lambda.getProject(), lambda.getId(), lambda.getEtag(), lambda.getInstanceNumber());
            }
            catch (e) {
                throw new Error(e.toString());
            }
        }
        throw new TypeError("URI is invalid data type.");
    }
}
exports.LambdaURI = LambdaURI;
